@@ -60,8 +60,7 @@ from .types import \
     StructMeta
 
 from ._gobject._gobject import \
-    GInterface, \
-    GObject
+    GInterface
 
 from ._gobject.constants import \
     TYPE_NONE, \
@@ -81,10 +80,16 @@ def get_parent_for_object(object_info):
     parent_object_info = object_info.get_parent()
 
     if not parent_object_info:
-        # Special case GObject.Object as being derived from the static GObject.
-        if object_info.get_namespace() == 'GObject' and object_info.get_name() == 'Object':
-            return GObject
+        # If we reach the end of the introspection info class hierarchy, look
+        # for an existing wrapper on the GType and use it as a base for the
+        # new introspection wrapper. This allows static C wrappers already
+        # registered with the GType to be used as the introspection base
+        # (_gobject.GObject for example)
+        gtype = object_info.get_g_type()
+        if gtype and gtype.pytype:
+            return gtype.pytype
 
+        # Otherwise use builtins.object as the base
         return object
 
     namespace = parent_object_info.get_namespace()
@@ -163,6 +168,8 @@ class IntrospectionModule(object):
                 for value_info in info.get_values():
                     value_name = value_info.get_name_unescaped().translate(ascii_upper_trans)
                     setattr(wrapper, value_name, wrapper(value_info.get_value()))
+                for method_info in info.get_methods():
+                    setattr(wrapper, method_info.__name__, method_info)
 
             if g_type != TYPE_NONE:
                 g_type.pytype = wrapper
