@@ -36,7 +36,8 @@ typedef gboolean (*PyGIMarshalFromPyFunc) (PyGIInvokeState   *state,
                                            PyGICallableCache *callable_cache,
                                            PyGIArgCache      *arg_cache,
                                            PyObject          *py_arg,
-                                           GIArgument        *arg);
+                                           GIArgument        *arg,
+                                           gpointer          *cleanup_data);
 
 typedef PyObject *(*PyGIMarshalToPyFunc) (PyGIInvokeState   *state,
                                           PyGICallableCache *callable_cache,
@@ -45,6 +46,7 @@ typedef PyObject *(*PyGIMarshalToPyFunc) (PyGIInvokeState   *state,
 
 typedef void (*PyGIMarshalCleanupFunc) (PyGIInvokeState *state,
                                         PyGIArgCache    *arg_cache,
+                                        PyObject        *py_arg, /* always NULL for to_py cleanup */
                                         gpointer         data,
                                         gboolean         was_processed);
 
@@ -56,10 +58,6 @@ typedef void (*PyGIMarshalCleanupFunc) (PyGIInvokeState *state,
  *  - PYGI_META_ARG_TYPE_CHILD - Children without python argument are
  *    ignored by the marshallers and handled directly by their parents
  *    marshaller.
- *  - PYGI_META_ARG_TYPE_CHILD_NEEDS_UPDATE -Sometimes children arguments
- *    come before the parent.  In these cases they need to be flagged
- *    so that the argument list counts must be updated for the cache to
- *    be valid
  *  - Children with pyargs (PYGI_META_ARG_TYPE_CHILD_WITH_PYARG) are processed
  *    the same as other child args but also have an index into the 
  *    python parameters passed to the invoker
@@ -67,7 +65,6 @@ typedef void (*PyGIMarshalCleanupFunc) (PyGIInvokeState *state,
 typedef enum {
     PYGI_META_ARG_TYPE_PARENT,
     PYGI_META_ARG_TYPE_CHILD,
-    PYGI_META_ARG_TYPE_CHILD_NEEDS_UPDATE,
     PYGI_META_ARG_TYPE_CHILD_WITH_PYARG,
     PYGI_META_ARG_TYPE_CLOSURE,
 } PyGIMetaArgType;
@@ -95,9 +92,9 @@ typedef enum {
  * marshalled into Python or from Python.
  */
 typedef enum {
-    PYGI_DIRECTION_TO_PYTHON,
-    PYGI_DIRECTION_FROM_PYTHON,
-    PYGI_DIRECTION_BIDIRECTIONAL
+    PYGI_DIRECTION_TO_PYTHON     = 1 << 0,
+    PYGI_DIRECTION_FROM_PYTHON   = 1 << 1,
+    PYGI_DIRECTION_BIDIRECTIONAL = PYGI_DIRECTION_TO_PYTHON | PYGI_DIRECTION_FROM_PYTHON
  } PyGIDirection;
 
 
@@ -110,6 +107,7 @@ struct _PyGIArgCache
     gboolean is_caller_allocates;
     gboolean is_skipped;
     gboolean allow_none;
+    gboolean has_default;
 
     PyGIDirection direction;
     GITransfer transfer;
@@ -126,6 +124,9 @@ struct _PyGIArgCache
 
     gssize c_arg_index;
     gssize py_arg_index;
+
+    /* Set when has_default is true. */
+    GIArgument default_value;
 };
 
 typedef struct _PyGISequenceCache
@@ -172,18 +173,34 @@ struct _PyGICallableCache
     PyGIFunctionType function_type;
 
     PyGIArgCache *return_cache;
-    PyGIArgCache **args_cache;
+    GPtrArray *args_cache;
     GSList *to_py_args;
     GSList *arg_name_list; /* for keyword arg matching */
     GHashTable *arg_name_hash;
 
-    /* counts */
+    /* Index of user_data arg that can eat variable args passed to a callable. */
+    gssize user_data_varargs_index;
+
+    /* Number of in args passed to g_function_info_invoke.
+     * This is used for the length of PyGIInvokeState.in_args */
     gssize n_from_py_args;
+
+    /* Number of out args passed to g_function_info_invoke.
+     * This is used for the length of PyGIInvokeState.out_values */
     gssize n_to_py_args;
+
+    /* Number of out args for g_function_info_invoke that will be skipped
+     * when marshaling to Python due to them being implicitly available
+     * (list/array length).
+     */
     gssize n_to_py_child_args;
 
-    gssize n_args;
+    /* Number of Python arguments expected for invoking the gi function. */
     gssize n_py_args;
+
+    /* Minimum number of args required to call the callable from Python.
+     * This count does not include args with defaults. */
+    gssize n_py_required_args;
 };
 
 void _pygi_arg_cache_clear	(PyGIArgCache *cache);
@@ -192,6 +209,18 @@ void _pygi_callable_cache_free	(PyGICallableCache *cache);
 PyGICallableCache *_pygi_callable_cache_new (GICallableInfo *callable_info,
                                              gboolean is_ccallback);
 
+#define _pygi_callable_cache_args_len(cache) ((cache)->args_cache)->len
+
+inline static PyGIArgCache *
+_pygi_callable_cache_get_arg (PyGICallableCache *cache, guint index) {
+    return (PyGIArgCache *) g_ptr_array_index (cache->args_cache, index);
+}
+
+inline static void
+_pygi_callable_cache_set_arg (PyGICallableCache *cache, guint index, PyGIArgCache *arg_cache) {
+    cache->args_cache->pdata[index] = arg_cache;
+}
+
 G_END_DECLS
 
 #endif /* __PYGI_CACHE_H__ */
