@@ -369,11 +369,33 @@ _pygi_closure_convert_arguments (GICallableInfo *callable_info, void **args,
                     g_type_info_is_pointer (&arg_type)) {
 
                 if (user_data == NULL) {
+                    /* user_data can be NULL for connect functions which don't accept
+                     * user_data or as the default for user_data in the middle of function
+                     * arguments.
+                     */
                     Py_INCREF (Py_None);
                     value = Py_None;
                 } else {
-                    value = user_data;
-                    Py_INCREF (value);
+                    /* Extend the callbacks args with user_data as variable args. */
+                    int j, user_data_len;
+                    PyObject *py_user_data = user_data;
+
+                    if (!PyTuple_Check (py_user_data)) {
+                        PyErr_SetString (PyExc_TypeError, "expected tuple for callback user_data");
+                        goto error;
+                    }
+
+                    user_data_len = PyTuple_Size (py_user_data);
+                    _PyTuple_Resize (py_args, n_args + user_data_len - 1);
+                    for (j = 0; j < user_data_len; j++, n_in_args++) {
+                        value = PyTuple_GetItem (py_user_data, j);
+                        Py_INCREF (value);
+                        PyTuple_SET_ITEM (*py_args, n_in_args, value);
+                    }
+                    /* We can assume user_data args are never going to be inout,
+                     * so just continue here.
+                     */
+                    continue;
                 }
             } else if (direction == GI_DIRECTION_IN &&
                        arg_tag == GI_TYPE_TAG_INTERFACE) {
@@ -631,10 +653,10 @@ _pygi_make_native_closure (GICallableInfo* info,
     closure = g_slice_new0 (PyGICClosure);
     closure->info = (GICallableInfo *) g_base_info_ref ( (GIBaseInfo *) info);
     closure->function = py_function;
-    closure->user_data = py_user_data ? py_user_data : Py_None;
+    closure->user_data = py_user_data;
 
     Py_INCREF (py_function);
-    Py_INCREF (closure->user_data);
+    Py_XINCREF (closure->user_data);
 
     fficlosure =
         g_callable_info_prepare_closure (info, &closure->cif, _pygi_closure_handle,
