@@ -24,6 +24,7 @@
 static inline void
 _cleanup_caller_allocates (PyGIInvokeState    *state,
                            PyGIArgCache       *cache,
+                           PyObject           *py_obj,
                            gpointer            data,
                            gboolean            was_processed)
 {
@@ -93,20 +94,23 @@ pygi_marshal_cleanup_args_from_py_marshal_success (PyGIInvokeState   *state,
 {
     gssize i;
 
-    /* For in success, call cleanup for all GI_DIRECTION_IN values only. */
-    for (i = 0; i < cache->n_args; i++) {
-        PyGIArgCache *arg_cache = cache->args_cache[i];
+    for (i = 0; i < _pygi_callable_cache_args_len (cache); i++) {
+        PyGIArgCache *arg_cache = _pygi_callable_cache_get_arg (cache, i);
         PyGIMarshalCleanupFunc cleanup_func = arg_cache->from_py_cleanup;
+        PyObject *py_arg = PyTuple_GET_ITEM (state->py_in_args,
+                                             arg_cache->py_arg_index);
+        gpointer cleanup_data = state->args_cleanup_data[i];
 
-        if (cleanup_func &&
-                arg_cache->direction == PYGI_DIRECTION_FROM_PYTHON &&
-                    state->args[i]->v_pointer != NULL)
-            cleanup_func (state, arg_cache, state->args[i]->v_pointer, TRUE);
-
-        if (cleanup_func &&
-                arg_cache->direction == PYGI_DIRECTION_BIDIRECTIONAL &&
-                    state->args_data[i] != NULL) {
-            cleanup_func (state, arg_cache, state->args_data[i], TRUE);
+        /* Only cleanup using args_cleanup_data when available.
+         * It is the responsibility of the various "from_py" marshalers to return
+         * cleanup_data which is then passed into their respective cleanup function.
+         * PyGIInvokeState.args_cleanup_data stores this data (via _invoke_marshal_in_args)
+         * for the duration of the invoke up until this point.
+         */
+        if (cleanup_func && cleanup_data != NULL &&
+                arg_cache->direction & PYGI_DIRECTION_FROM_PYTHON) {
+            cleanup_func (state, arg_cache, py_arg, cleanup_data, TRUE);
+            state->args_cleanup_data[i] = NULL;
         }
     }
 }
@@ -122,6 +126,7 @@ pygi_marshal_cleanup_args_to_py_marshal_success (PyGIInvokeState   *state,
         if (cleanup_func && state->return_arg.v_pointer != NULL)
             cleanup_func (state,
                           cache->return_cache,
+                          NULL,
                           state->return_arg.v_pointer,
                           TRUE);
     }
@@ -136,11 +141,13 @@ pygi_marshal_cleanup_args_to_py_marshal_success (PyGIInvokeState   *state,
         if (cleanup_func != NULL && data != NULL)
             cleanup_func (state,
                           arg_cache,
+                          NULL,
                           data,
                           TRUE);
         else if (arg_cache->is_caller_allocates && data != NULL) {
             _cleanup_caller_allocates (state,
                                        arg_cache,
+                                       NULL,
                                        data,
                                        TRUE);
         }
@@ -158,22 +165,26 @@ pygi_marshal_cleanup_args_from_py_parameter_fail (PyGIInvokeState   *state,
 
     state->failed = TRUE;
 
-    for (i = 0; i < cache->n_args  && i <= failed_arg_index; i++) {
-        PyGIArgCache *arg_cache = cache->args_cache[i];
+    for (i = 0; i < _pygi_callable_cache_args_len (cache)  && i <= failed_arg_index; i++) {
+        PyGIArgCache *arg_cache = _pygi_callable_cache_get_arg (cache, i);
         PyGIMarshalCleanupFunc cleanup_func = arg_cache->from_py_cleanup;
         gpointer data = state->args[i]->v_pointer;
+        PyObject *py_arg = PyTuple_GET_ITEM (state->py_in_args,
+                                             arg_cache->py_arg_index);
 
         if (cleanup_func &&
                 arg_cache->direction == PYGI_DIRECTION_FROM_PYTHON &&
                     data != NULL) {
             cleanup_func (state,
                           arg_cache,
+                          py_arg,
                           data,
                           i < failed_arg_index);
 
         } else if (arg_cache->is_caller_allocates && data != NULL) {
             _cleanup_caller_allocates (state,
                                        arg_cache,
+                                       py_arg,
                                        data,
                                        FALSE);
         }
@@ -198,18 +209,19 @@ pygi_marshal_cleanup_args_to_py_parameter_fail (PyGIInvokeState   *state,
 void
 _pygi_marshal_cleanup_from_py_utf8 (PyGIInvokeState *state,
                                     PyGIArgCache    *arg_cache,
+                                    PyObject        *py_arg,
                                     gpointer         data,
                                     gboolean         was_processed)
 {
-    /* We strdup strings so always free if we have processed this
-       parameter for input */
-    if (was_processed)
+    /* We strdup strings so free unless ownership is transferred to C. */
+    if (was_processed && arg_cache->transfer == GI_TRANSFER_NOTHING)
         g_free (data);
 }
 
 void
 _pygi_marshal_cleanup_to_py_utf8 (PyGIInvokeState *state,
                                   PyGIArgCache    *arg_cache,
+                                  PyObject        *dummy,
                                   gpointer         data,
                                   gboolean         was_processed)
 {
@@ -223,6 +235,7 @@ _pygi_marshal_cleanup_to_py_utf8 (PyGIInvokeState *state,
 void
 _pygi_marshal_cleanup_from_py_interface_object (PyGIInvokeState *state,
                                                 PyGIArgCache    *arg_cache,
+                                                PyObject        *py_arg,
                                                 gpointer         data,
                                                 gboolean         was_processed)
 {
@@ -236,6 +249,7 @@ _pygi_marshal_cleanup_from_py_interface_object (PyGIInvokeState *state,
 void
 _pygi_marshal_cleanup_to_py_interface_object (PyGIInvokeState *state,
                                               PyGIArgCache    *arg_cache,
+                                              PyObject        *dummy,
                                               gpointer         data,
                                               gboolean         was_processed)
 {
@@ -249,28 +263,31 @@ _pygi_marshal_cleanup_to_py_interface_object (PyGIInvokeState *state,
 void
 _pygi_marshal_cleanup_from_py_interface_callback (PyGIInvokeState *state,
                                                   PyGIArgCache    *arg_cache,
+                                                  PyObject        *py_arg,
                                                   gpointer         data,
                                                   gboolean         was_processed)
 {
     PyGICallbackCache *callback_cache = (PyGICallbackCache *)arg_cache;
     if (was_processed && callback_cache->scope == GI_SCOPE_TYPE_CALL) {
-        _pygi_invoke_closure_free (state->args_data[arg_cache->c_arg_index]);
-        state->args_data[arg_cache->c_arg_index] = NULL;
+        _pygi_invoke_closure_free (data);
     }
 }
 
 void 
 _pygi_marshal_cleanup_from_py_interface_struct_gvalue (PyGIInvokeState *state,
                                                        PyGIArgCache    *arg_cache,
+                                                       PyObject        *py_arg,
                                                        gpointer         data,
                                                        gboolean         was_processed)
 {
-    if (was_processed) {
-        PyObject *py_arg = PyTuple_GET_ITEM (state->py_in_args,
-                                             arg_cache->py_arg_index);
+    /* Note py_arg can be NULL for hash table which is a bug. */
+    if (was_processed && py_arg != NULL) {
         GType py_object_type =
             pyg_type_from_object_strict ( (PyObject *) py_arg->ob_type, FALSE);
 
+        /* When a GValue was not passed, it means the marshalers created a new
+         * one to pass in, clean this up.
+         */
         if (py_object_type != G_TYPE_VALUE) {
             g_value_unset ((GValue *) data);
             g_slice_free (GValue, data);
@@ -281,6 +298,7 @@ _pygi_marshal_cleanup_from_py_interface_struct_gvalue (PyGIInvokeState *state,
 void
 _pygi_marshal_cleanup_from_py_interface_struct_foreign (PyGIInvokeState *state,
                                                         PyGIArgCache    *arg_cache,
+                                                        PyObject        *py_arg,
                                                         gpointer         data,
                                                         gboolean         was_processed)
 {
@@ -293,6 +311,7 @@ _pygi_marshal_cleanup_from_py_interface_struct_foreign (PyGIInvokeState *state,
 void
 _pygi_marshal_cleanup_to_py_interface_struct_foreign (PyGIInvokeState *state,
                                                       PyGIArgCache    *arg_cache,
+                                                      PyObject        *dummy,
                                                       gpointer         data,
                                                       gboolean         was_processed)
 {
@@ -336,6 +355,7 @@ _wrap_c_array (PyGIInvokeState   *state,
 void
 _pygi_marshal_cleanup_from_py_array (PyGIInvokeState *state,
                                      PyGIArgCache    *arg_cache,
+                                     PyObject        *py_arg,
                                      gpointer         data,
                                      gboolean         was_processed)
 {
@@ -344,15 +364,7 @@ _pygi_marshal_cleanup_from_py_array (PyGIInvokeState *state,
         GPtrArray *ptr_array_ = NULL;
         PyGISequenceCache *sequence_cache = (PyGISequenceCache *)arg_cache;
 
-        /* If this isn't a garray create one to help process variable sized
-           array elements */
-        if (sequence_cache->array_type == GI_ARRAY_TYPE_C) {
-            array_ = _wrap_c_array (state, sequence_cache, data);
-            
-            if (array_ == NULL)
-                return;
-            
-        } else if (sequence_cache->array_type == GI_ARRAY_TYPE_PTR_ARRAY) {
+        if (sequence_cache->array_type == GI_ARRAY_TYPE_PTR_ARRAY) {
             ptr_array_ = (GPtrArray *) data;
         } else {
             array_ = (GArray *) data;
@@ -367,6 +379,7 @@ _pygi_marshal_cleanup_from_py_array (PyGIInvokeState *state,
 
             for (i = 0; i < len; i++) {
                 gpointer item;
+                PyObject *py_item = NULL;
 
                 /* case 1: GPtrArray */
                 if (ptr_array_ != NULL)
@@ -387,19 +400,23 @@ _pygi_marshal_cleanup_from_py_array (PyGIInvokeState *state,
                     }
                 }
 
-                cleanup_func (state, sequence_cache->item_cache, item, TRUE);
+                py_item = PySequence_GetItem (py_arg, i);
+                cleanup_func (state, sequence_cache->item_cache, py_item, item, TRUE);
+                Py_XDECREF (py_item);
             }
         }
 
         /* Only free the array when we didn't transfer ownership */
         if (sequence_cache->array_type == GI_ARRAY_TYPE_C) {
+            /* always free the GArray wrapper created in from_py marshaling and
+             * passed back as cleanup_data
+             */
             g_array_free (array_, arg_cache->transfer == GI_TRANSFER_NOTHING);
-        } else if (state->failed ||
-                   arg_cache->transfer == GI_TRANSFER_NOTHING) {
+        } else {
             if (array_ != NULL)
-                g_array_free (array_, TRUE);
+                g_array_unref (array_);
             else
-                g_ptr_array_free (ptr_array_, TRUE);
+                g_ptr_array_unref (ptr_array_);
         }
     }
 }
@@ -407,6 +424,7 @@ _pygi_marshal_cleanup_from_py_array (PyGIInvokeState *state,
 void
 _pygi_marshal_cleanup_to_py_array (PyGIInvokeState *state,
                                    PyGIArgCache    *arg_cache,
+                                   PyObject        *dummy,
                                    gpointer         data,
                                    gboolean         was_processed)
 {
@@ -438,6 +456,7 @@ _pygi_marshal_cleanup_to_py_array (PyGIInvokeState *state,
             for (i = 0; i < len; i++) {
                 cleanup_func (state,
                               sequence_cache->item_cache,
+                              NULL,
                               (array_ != NULL) ? g_array_index (array_, gpointer, i) : g_ptr_array_index (ptr_array_, i),
                               was_processed);
             }
@@ -453,6 +472,7 @@ _pygi_marshal_cleanup_to_py_array (PyGIInvokeState *state,
 void
 _pygi_marshal_cleanup_from_py_glist  (PyGIInvokeState *state,
                                       PyGIArgCache    *arg_cache,
+                                      PyObject        *py_arg,
                                       gpointer         data,
                                       gboolean         was_processed)
 {
@@ -467,28 +487,26 @@ _pygi_marshal_cleanup_from_py_glist  (PyGIInvokeState *state,
             PyGIMarshalCleanupFunc cleanup_func =
                 sequence_cache->item_cache->from_py_cleanup;
             GSList *node = list_;
+            gsize i = 0;
             while (node != NULL) {
+                PyObject *py_item = PySequence_GetItem (py_arg, i);
                 cleanup_func (state,
                               sequence_cache->item_cache,
+                              py_item,
                               node->data,
                               TRUE);
+                Py_XDECREF (py_item);
                 node = node->next;
+                i++;
             }
         }
 
-        if (state->failed ||
-               arg_cache->transfer == GI_TRANSFER_NOTHING ||
-                  arg_cache->transfer == GI_TRANSFER_CONTAINER) {
-            switch (arg_cache->type_tag) {
-                case GI_TYPE_TAG_GLIST:
-                    g_list_free ( (GList *)list_);
-                    break;
-                case GI_TYPE_TAG_GSLIST:
-                    g_slist_free (list_);
-                    break;
-                default:
-                    g_assert_not_reached();
-            }
+        if (arg_cache->type_tag == GI_TYPE_TAG_GLIST) {
+            g_list_free ( (GList *)list_);
+        } else if (arg_cache->type_tag == GI_TYPE_TAG_GSLIST) {
+            g_slist_free (list_);
+        } else {
+            g_assert_not_reached();
         }
     }
 }
@@ -496,11 +514,11 @@ _pygi_marshal_cleanup_from_py_glist  (PyGIInvokeState *state,
 void
 _pygi_marshal_cleanup_to_py_glist (PyGIInvokeState *state,
                                    PyGIArgCache    *arg_cache,
+                                   PyObject        *dummy,
                                    gpointer         data,
                                    gboolean         was_processed)
 {
     PyGISequenceCache *sequence_cache = (PyGISequenceCache *)arg_cache;
-
     if (arg_cache->transfer == GI_TRANSFER_EVERYTHING ||
             arg_cache->transfer == GI_TRANSFER_CONTAINER) {
         GSList *list_ = (GSList *)data;
@@ -513,23 +531,19 @@ _pygi_marshal_cleanup_to_py_glist (PyGIInvokeState *state,
             while (node != NULL) {
                 cleanup_func (state,
                               sequence_cache->item_cache,
+                              NULL,
                               node->data,
                               was_processed);
                 node = node->next;
             }
         }
 
-        if (arg_cache->transfer == GI_TRANSFER_EVERYTHING) {
-            switch (arg_cache->type_tag) {
-                case GI_TYPE_TAG_GLIST:
-                    g_list_free ( (GList *)list_);
-                    break;
-                case GI_TYPE_TAG_GSLIST:
-                    g_slist_free (list_);
-                    break;
-                default:
-                    g_assert_not_reached();
-            }
+        if (arg_cache->type_tag == GI_TYPE_TAG_GLIST) {
+            g_list_free ( (GList *)list_);
+        } else if (arg_cache->type_tag == GI_TYPE_TAG_GSLIST) {
+            g_slist_free (list_);
+        } else {
+            g_assert_not_reached();
         }
     }
 }
@@ -537,6 +551,7 @@ _pygi_marshal_cleanup_to_py_glist (PyGIInvokeState *state,
 void
 _pygi_marshal_cleanup_from_py_ghash  (PyGIInvokeState *state,
                                       PyGIArgCache    *arg_cache,
+                                      PyObject        *py_arg,
                                       gpointer         data,
                                       gboolean         was_processed)
 {
@@ -566,27 +581,26 @@ _pygi_marshal_cleanup_from_py_ghash  (PyGIInvokeState *state,
                 if (key != NULL && key_cleanup_func != NULL)
                     key_cleanup_func (state,
                                       hash_cache->key_cache,
+                                      NULL,
                                       key,
                                       TRUE);
                 if (value != NULL && value_cleanup_func != NULL)
                     value_cleanup_func (state,
                                         hash_cache->value_cache,
+                                        NULL,
                                         value,
                                         TRUE);
             }
         }
 
-        if (state->failed ||
-               arg_cache->transfer == GI_TRANSFER_NOTHING ||
-                  arg_cache->transfer == GI_TRANSFER_CONTAINER)
-            g_hash_table_destroy (hash_);
-
+        g_hash_table_unref (hash_);
     }
 }
 
 void
 _pygi_marshal_cleanup_to_py_ghash (PyGIInvokeState *state,
                                    PyGIArgCache    *arg_cache,
+                                   PyObject        *dummy,
                                    gpointer         data,
                                    gboolean         was_processed)
 {
@@ -594,6 +608,6 @@ _pygi_marshal_cleanup_to_py_ghash (PyGIInvokeState *state,
         return;
 
     /* assume hashtable has boxed key and value */
-    if (arg_cache->transfer == GI_TRANSFER_EVERYTHING)
-        g_hash_table_destroy ( (GHashTable *)data);
+    if (arg_cache->transfer == GI_TRANSFER_EVERYTHING || arg_cache->transfer == GI_TRANSFER_CONTAINER)
+        g_hash_table_unref ( (GHashTable *)data);
 }
